/**
 * 
 */
package org.springblade.report.enums;

import org.springblade.cube.enums.DbClickHouseEnums;
import org.springblade.cube.enums.DbKingbaseEnums;
import org.springblade.cube.enums.DbMysqlEnums;
import org.springblade.cube.enums.DbOracleEnums;
import org.springblade.cube.enums.DbSqlServerEnums;

import com.baomidou.mybatisplus.annotation.EnumValue;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author wookvn
 *
 */
@Getter
@AllArgsConstructor
public enum DsTypeEnum {

	mysql(0, "com.mysql.cj.jdbc.Driver", "select 1", DbMysqlEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:mysql://%s:%s/%s", dsIp, dsPort, dsDb) + "?rewriteBatchedStatements=true&serverTimezone=GMT%2B8";
		}
	},
	
	oracle(1, "oracle.jdbc.OracleDriver", "select 1 from dual", DbOracleEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:oracle:thin:@%s:%s/%s", dsIp, dsPort, dsDb);
		}
	},
	
	sqlserver(2, "com.microsoft.sqlserver.jdbc.SQLServerDriver", "select 1", DbSqlServerEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:sqlserver://%s:%s;DatabaseName=%s", dsIp, dsPort, dsDb);
		}
	},
	
	postgresql(3, "org.postgresql.Driver", "select 1", 3000){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:postgresql://%s:%s/%s", dsIp, dsPort, dsDb);
		}
	},
	
	clickhouse(4, "com.clickhouse.jdbc.ClickHouseDriver", "select 1", DbClickHouseEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:clickhouse://%s:%s/%s", dsIp, dsPort, dsDb);
		}
	},
	
	kingbase(5, "com.kingbase8.Driver", "select 1", DbKingbaseEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:kingbase8://%s:%s/%s?currentSchema=public", dsIp, dsPort, dsDb);
		}
	},
	
	//通用版本
	common(-1, "common", "common", 0){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return null;
		}
	};
	
	@EnumValue
	private Integer value;
	private String driverClassName;
	private String validationQuery;
	private int batchSize;  //批量入库数量

	public abstract String createUrl(String dsIp, Integer dsPort, String dsDb);
	
	/**
	 * 解析
	 * @param value
	 * @return
	 */
	public static DsTypeEnum resolve(Integer value) {
		DsTypeEnum[] values = DsTypeEnum.values();
		for(DsTypeEnum dsType : values) {
			if(value != null && dsType.getValue().intValue() == value) {
				return dsType;
			}
		}
		return DsTypeEnum.mysql;
	}
}
