/**
 * 
 */
package org.springblade.visual.controller;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualPowerPointCategory;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualPowerPointCategoryService;
import org.springblade.visual.service.VisualPowerPointInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 *	幻灯片 操作处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@RestController
@RequestMapping("visual/powerPointInfo")
@Api(value = "幻灯片相关接口", tags = "幻灯片接口")
public class VisualPowerPointInfoController extends BladeController {

	@Autowired
	private VisualDataService visualDataService;
	
	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private VisualPowerPointInfoService visualPowerPointInfoService;
	
	@Autowired
	private VisualPowerPointCategoryService visualPowerPointCategoryService;
	
	/**
	 * 	列表数据
	 * @param visualPowerPointInfo
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页查询幻灯片", httpMethod = "GET")
	public R<IPage<VisualPowerPointInfo>> pageList(VisualPowerPointInfo visualPowerPointInfo, Query query){
		QueryWrapper<VisualPowerPointInfo> queryWrapper = Condition.getQueryWrapper(visualPowerPointInfo);
		IPage<VisualPowerPointInfo> pageResult = visualPowerPointInfoService.page(Condition.getPage(query), queryWrapper.lambda().orderByDesc(VisualPowerPointInfo::getId));
		if(pageResult != null && CollectionUtil.isNotEmpty(pageResult.getRecords())) {
			List<Integer> ppidList = pageResult.getRecords().stream().map(VisualPowerPointInfo::getId).collect(Collectors.toList());
			List<VisualPowerPointInfo> datalist = visualPowerPointInfoService.listByPpidGroup(ppidList);
			if(CollectionUtil.isNotEmpty(datalist)) {
				Map<Integer, Integer> dataMap = datalist.stream().collect(Collectors.toMap(VisualPowerPointInfo::getId, VisualPowerPointInfo::getVisualCount));
				pageResult.getRecords().forEach(item -> item.setVisualCount(dataMap.getOrDefault(item.getId(), 0)));
			}
		}
		return R.data(pageResult);
	}
	
	/**
	 * 单条数据查询接口
	 * @param id
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "获取幻灯片详情以及大屏详情", httpMethod = "GET")
	public R<Kv> info(@RequestParam Integer id) {
		LambdaQueryWrapper<VisualData> queryWrapper = Wrappers.<VisualData>lambdaQuery().eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT)
				.eq(VisualData::getPowerPointId, id).orderByAsc(VisualData::getOrderIndex);
		List<VisualData> visualList = visualDataService.list(queryWrapper);
		visualList.forEach(item -> {
			//查询大屏使用的所有组件
			LambdaQueryWrapper<VisualComponent> _queryWrapper = Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId, item.getId());
			List<Integer> componentIdList = visualComponentService.listObjs(_queryWrapper, a -> Integer.valueOf(a.toString()));
			item.setComponentIds(componentIdList);
		});
		Kv detail = Kv.create();
		VisualPowerPointInfo powerPointInfo = visualPowerPointInfoService.getById(id);
		//设置大屏数量
		powerPointInfo.setVisualCount(visualList.size());
		return R.data(detail.set("detail", powerPointInfo).set("visualList", visualList));
	}
	
	/**
	 *	修改数据接口
	 * @param visualPowerPointInfo
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改幻灯片基本信息", httpMethod = "POST")
	public R<String> doUpdate(@Validated @RequestBody VisualPowerPointInfo visualPowerPointInfo) {
		if(visualPowerPointInfo.getId() == null) {
			visualPowerPointInfo.setCreateTime(LocalDateTime.now());
			visualPowerPointInfo.setCreateUserId(getUser().getUserId());
		}
		visualPowerPointInfo.setUpdateBy(getUser().getNickName());
		visualPowerPointInfo.setUpdateTime(LocalDateTime.now());
		//判断分类是否父级分类，父级分类不能放置幻灯片
		int count = visualPowerPointCategoryService.count(Wrappers.<VisualPowerPointCategory>lambdaQuery().eq(VisualPowerPointCategory::getParentId, visualPowerPointInfo.getCateId()));
		if(count > 0) {
			return R.fail("该分类下有子分类，不能放置幻灯片。");
		}
		return R.status(visualPowerPointInfoService.saveOrUpdate(visualPowerPointInfo));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除幻灯片", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer id) {
		return R.status(visualPowerPointInfoService.doRemove(id));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/addVisual")
	@ApiOperation(value = "添加幻灯片的大屏列表", httpMethod = "POST")
	public R<List<Integer>> doUpdateVisual(
			@ApiParam(value = "幻灯片ID", required = true) @RequestParam Integer ppId,
			@ApiParam(value = "大屏ID列表，多个逗号隔开", required = true) @RequestParam String visualIds) {
		return R.data(visualPowerPointInfoService.doAddVisual(ppId, visualIds, getUser()));
	}
	
	/**
	 * 删除幻灯片中的某一个大屏
	 * @param ppId
	 * @param visualId
	 * @return
	 */
	@PostMapping("/deleteVisual")
	@ApiOperation(value = "删除幻灯片的单张大屏", httpMethod = "POST")
	public R<String> doDeleteVisual(
			@ApiParam(value = "幻灯片ID", required = true) @RequestParam Integer ppId,
			@ApiParam(value = "大屏ID", required = true) @RequestParam Integer visualId){
		return R.status(visualPowerPointInfoService.doRemoveVisual(ppId, visualId, getUser()));
	}
	
}