/**
 * 
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualPowerPointInfoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 *	幻灯片 服务处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@Service
public class VisualPowerPointInfoService extends ServiceImpl<VisualPowerPointInfoMapper, VisualPowerPointInfo>{
	
	@Autowired
	private VisualDataService visualDataService;
	
	/**
	 * 
	 * @param ppidList
	 * @return
	 */
	public List<VisualPowerPointInfo> listByPpidGroup(List<Integer> ppidList){
		return baseMapper.listByPpidGroup(ppidList);
	}
	
	/**
	 * 根据ID删除一个幻灯片
	 * @param id
	 * @return
	 */
	@Transactional
	public boolean doRemove(Integer id) {
		LambdaQueryWrapper<VisualData> queryWrapper = Wrappers.<VisualData>lambdaQuery().select(VisualData::getId).eq(VisualData::getPowerPointId, id).eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT);
		List<Integer> visualIdList = visualDataService.listObjs(queryWrapper, a -> Integer.valueOf(a.toString()));
		if(CollectionUtil.isNotEmpty(visualIdList)) {  //删除大屏
			visualDataService.doRemove(visualIdList);
		}
		return baseMapper.deleteById(id) > 0;
	}
	
	/**
	 * 添加幻灯片的大屏列表
	 * @param ppId
	 * @param visualIds
	 * @return
	 */
	@Transactional
	public List<Integer> doAddVisual(Integer ppId, String visualIds, BladeUser bladeUser) {
		Set<Integer> visualIdSet = new HashSet<Integer>(Func.toIntList(visualIds));
		List<Integer> newVisualIdList = null;
		if(CollectionUtil.isNotEmpty(visualIdSet)) { //根据原大屏ID复制一份新的大屏
			newVisualIdList = visualDataService.doCopyPowerPoint(visualIdSet, ppId, bladeUser);
			//更新时间
			VisualPowerPointInfo entity = new VisualPowerPointInfo();
			entity.setId(ppId);
			entity.setUpdateBy(bladeUser.getNickName());
			entity.setUpdateTime(LocalDateTime.now());
			baseMapper.updateById(entity);
		}
		return newVisualIdList;
	}
	
	/**
	 * 删除单张大屏
	 * @param ppId
	 * @param visualId
	 * @return
	 */
	@Transactional
	public boolean doRemoveVisual(Integer ppId, Integer visualId, BladeUser bladeUser) {
		boolean ret = visualDataService.remove(Wrappers.<VisualData>lambdaQuery().eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT).eq(VisualData::getPowerPointId, ppId).eq(VisualData::getId, visualId));
		if(ret) {
			VisualPowerPointInfo entity = new VisualPowerPointInfo();
			entity.setId(ppId);
			entity.setUpdateBy(bladeUser.getNickName());
			entity.setUpdateTime(LocalDateTime.now());
			baseMapper.updateById(entity);
		}
		return ret;
	}
}